"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.name = void 0;
const Service_1 = __importDefault(require("@sudobot/core/Service"));
const FileSystem_1 = __importDefault(require("@sudobot/polyfills/FileSystem"));
const download_1 = require("@sudobot/utils/download");
const embed_1 = require("@sudobot/utils/embed");
const fetch_1 = require("@sudobot/utils/fetch");
const utils_1 = require("@sudobot/utils/utils");
const discord_js_1 = require("discord.js");
exports.name = "urlfish";
class URLFishService extends Service_1.default {
    constructor() {
        super(...arguments);
        this.domainListURL = "https://raw.githubusercontent.com/mitchellkrogza/Phishing.Database/master/phishing-domains-ACTIVE.txt";
        this._list = [];
        this.patterns = [
            /\[steamcommunity\.com\/gift\/([^\/]+)\]\(.+?\)/gim
        ];
    }
    async boot() {
        const urlfishDir = (0, utils_1.sudoPrefix)("tmp/urlfish", true);
        const dataFile = (0, utils_1.sudoPrefix)("tmp/urlfish/LIST", false);
        if (await FileSystem_1.default.exists(dataFile)) {
            this.client.logger.debug("URLFishService", "Phishing domain list already exists", dataFile);
        }
        else {
            this.client.logger.debug("URLFishService", "Phishing domain list not found", dataFile);
            this.client.logger.debug("URLFishService", "Downloading list", dataFile);
            const url = this.domainListURL;
            await (0, download_1.downloadFile)({
                url,
                name: "LIST",
                path: urlfishDir
            });
        }
        const data = (await FileSystem_1.default.readFileContents(dataFile)).split("\n");
        this.client.logger.debug("URLFishService", `Loaded ${data.length} entries from file`);
        this._list = data;
    }
    get list() {
        return this._list;
    }
    scanMessage(message) {
        const urls = message.content.toLowerCase().split(" ");
        const phishingURLs = [];
        for (const url of urls) {
            const domain = url.startsWith("http") ? url.replace(/https?:\/?\/?/i, "") : url;
            if (this.list.includes(domain)) {
                phishingURLs.push(url);
            }
        }
        return phishingURLs;
    }
    async verifyMessage(message) {
        const config = this.client.configManager.get(message.guildId)?.urlfish;
        if (!config?.enabled ||
            (config.channels && "enabled_in" in config.channels && !config.channels.enabled_in.includes(message.channelId)) ||
            (config.channels && "disabled_in" in config.channels && config.channels.disabled_in.includes(message.channelId)) ||
            (await this.client.permissionManager.isImmuneToAutoMod(message.member))) {
            return;
        }
        for (const pattern of this.patterns) {
            if (pattern.test(message.content)) {
                await this.takeAction(message, config);
                await this.logMessage(message, config, [pattern.toString()], config.action);
                return;
            }
        }
        const links = this.scanMessage(message);
        if (links.length > 0) {
            await this.takeAction(message, config);
            await this.logMessage(message, config, links, config.action);
        }
    }
    async takeAction(message, config) {
        switch (config.action) {
            case "delete":
                if (message.deletable) {
                    await message.delete();
                }
                break;
            case "warn":
                await this.client.infractionManager.createMemberWarn(message.member, {
                    guild: message.guild,
                    reason: config.infraction_reason ??
                        "We have detected phishing URLs in your message. Please refrain from posting these links.",
                    moderator: this.client.user,
                    notifyUser: true,
                    sendLog: true
                });
                break;
            case "mute":
                await this.client.infractionManager.createMemberMute(message.member, {
                    guild: message.guild,
                    reason: config.infraction_reason ??
                        "We have detected phishing URLs in your message. Please refrain from posting these links.",
                    moderator: this.client.user,
                    notifyUser: true,
                    sendLog: true,
                    duration: config.mute_duration ?? 3_600_000, // 1 hour,
                    autoRemoveQueue: true
                });
                break;
            case "kick":
                await this.client.infractionManager.createMemberKick(message.member, {
                    guild: message.guild,
                    reason: config.infraction_reason ?? "We have detected phishing URLs in your message.",
                    moderator: this.client.user,
                    notifyUser: true,
                    sendLog: true
                });
                break;
            case "ban":
                await this.client.infractionManager.createUserBan(message.author, {
                    guild: message.guild,
                    reason: config.infraction_reason ?? "We have detected phishing URLs in your message.",
                    moderator: this.client.user,
                    notifyUser: true,
                    sendLog: true,
                    deleteMessageSeconds: 604_800, // 7 days,
                    autoRemoveQueue: true
                });
                break;
        }
    }
    async logMessage(message, config, links, action) {
        if (!config?.log_channel || !message.guild) {
            return;
        }
        const logChannel = await (0, fetch_1.safeChannelFetch)(message.guild, config?.log_channel);
        if (!logChannel?.isTextBased()) {
            return;
        }
        const joinedLinks = links.join("\n");
        await logChannel.send({
            embeds: [
                {
                    title: `Phishing URLs detected in ${message.url}`,
                    fields: [
                        {
                            name: "URLs",
                            value: joinedLinks.substring(0, 1020) + (joinedLinks.length > 1020 ? "..." : "")
                        },
                        {
                            name: "Channel",
                            value: (0, embed_1.channelInfo)(message.channel),
                            inline: true
                        },
                        {
                            name: "User",
                            value: (0, embed_1.userInfo)(message.author),
                            inline: true
                        },
                        {
                            name: "Action",
                            value: action === "ban"
                                ? "Banned"
                                : action[0].toUpperCase() + action.substring(1) + (action.endsWith("e") ? "d" : "ed"),
                            inline: true
                        }
                    ],
                    description: message.content || "*No content*",
                    color: discord_js_1.Colors.Red,
                    timestamp: message.createdAt.toISOString(),
                    author: {
                        name: message.author.username,
                        icon_url: message.author.displayAvatarURL()
                    },
                    footer: {
                        text: `Detected by URLFish`
                    }
                }
            ]
        });
    }
}
exports.default = URLFishService;
